local cql = ClassicQuestLog

--[[ timer for throttling events]]

cql.timer = CreateFrame("Frame")
cql.timer:Hide()

cql.timer.running = {} -- ordered list, the current functions waiting to be run
cql.timer.isRunning = {} -- lookup by function, whether this timer is running
cql.timer.times = {} -- lookup by function, the duration to wait before running the function

function cql.timer:Start(duration,func)
    self.isRunning[func] = true
    if not tContains(self.running,func) then
        tinsert(self.running,func)
        self.times[func] = duration
    end
    self:Show()
end

function cql.timer:Stop(func)
    if self.isRunning[func] then
        self.isRunning[func] = nil
        for i=#self.running,1,-1 do
            if self.running[i]==func then
                tremove(self.running,i)
            end
        end
    end
end

-- this only runs while cql.timer is shown
-- every frame, run through each running timer and see if it's ready to run, and run if so
cql.timer:SetScript("OnUpdate",function(self,elapsed)
    local tick = false
    for i=#self.running,1,-1 do
        local func = self.running[i]
        if func and self.times[func] then
            self.times[func] = self.times[func] - elapsed
            if self.times[func] < 0 then
                tremove(self.running,i)
                self.isRunning[func] = nil
                func()
            end
            tick = true
        end
    end
    if not tick then
        self:Hide()
    end
end)
